@@ -1,15 +1,14 @@
 import math
+
 import torch
 import torch.nn.functional as F
 
-__all__ = ['sigmoid_focal_loss', 'soft_jaccard_score', 'soft_dice_score', 'wing_loss']
+__all__ = ["sigmoid_focal_loss", "soft_jaccard_score", "soft_dice_score", "wing_loss"]
 
 
-def sigmoid_focal_loss(input: torch.Tensor,
-                       target: torch.Tensor,
-                       gamma=2.0,
-                       alpha=0.25,
-                       reduction='mean'):
+def sigmoid_focal_loss(
+    input: torch.Tensor, target: torch.Tensor, gamma=2.0, alpha=0.25, reduction="mean"
+):
     """Compute binary focal loss between target and output logits.
 
     See :class:`~pytorch_toolbelt.losses.FocalLoss` for details.
@@ -31,7 +30,7 @@ def sigmoid_focal_loss(input: torch.Tensor,
     """
     target = target.type(input.type())
 
-    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction='none')
+    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction="none")
     pt = torch.exp(logpt)
 
     # compute the loss
@@ -40,21 +39,23 @@ def sigmoid_focal_loss(input: torch.Tensor,
     if alpha is not None:
         loss = loss * (alpha * target + (1 - alpha) * (1 - target))
 
-    if reduction == 'mean':
+    if reduction == "mean":
         loss = loss.mean()
-    if reduction == 'sum':
+    if reduction == "sum":
         loss = loss.sum()
-    if reduction == 'batchwise_mean':
+    if reduction == "batchwise_mean":
         loss = loss.sum(0)
 
     return loss
 
 
-def reduced_focal_loss(input: torch.Tensor,
-                       target: torch.Tensor,
-                       threshold=0.5,
-                       gamma=2.0,
-                       reduction='mean'):
+def reduced_focal_loss(
+    input: torch.Tensor,
+    target: torch.Tensor,
+    threshold=0.5,
+    gamma=2.0,
+    reduction="mean",
+):
     """Compute reduced focal loss between target and output logits.
 
     See :class:`~pytorch_toolbelt.losses.FocalLoss` for details.
@@ -76,52 +77,93 @@ def reduced_focal_loss(input: torch.Tensor,
     """
     target = target.type(input.type())
 
-    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction='none')
+    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction="none")
     pt = torch.exp(logpt)
 
     # compute the loss
-    focal_reduction = ((1. - pt) / threshold).pow(gamma)
+    focal_reduction = ((1.0 - pt) / threshold).pow(gamma)
     focal_reduction[pt < threshold] = 1
 
-    loss = - focal_reduction * logpt
+    loss = -focal_reduction * logpt
 
-    if reduction == 'mean':
+    if reduction == "mean":
         loss = loss.mean()
-    if reduction == 'sum':
+    if reduction == "sum":
         loss = loss.sum()
-    if reduction == 'batchwise_mean':
+    if reduction == "batchwise_mean":
         loss = loss.sum(0)
 
     return loss
 
 
-def soft_jaccard_score(pred: torch.Tensor,
-                       target: torch.Tensor,
-                       smooth=1e-3,
-                       from_logits=False) -> torch.Tensor:
-    if from_logits:
-        pred = pred.sigmoid()
+def soft_jaccard_score(
+    y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0.0, eps=1e-7, dims=None
+) -> torch.Tensor:
+    """
 
-    target = target.float()
-    intersection = torch.sum(pred * target)
-    union = torch.sum(pred) + torch.sum(target)
-    iou = intersection / (union - intersection + smooth)
-    return iou
+    :param y_pred:
+    :param y_true:
+    :param smooth:
+    :param eps:
+    :return:
+
+    Shape:
+        - Input: :math:`(N, NC, *)` where :math:`*` means
+            any number of additional dimensions
+        - Target: :math:`(N, NC, *)`, same shape as the input
+        - Output: scalar.
+
+    """
+    assert y_pred.size() == y_true.size()
+
+    if dims is not None:
+        intersection = torch.sum(y_pred * y_true, dim=dims)
+        cardinality = torch.sum(y_pred + y_true, dim=dims)
+    else:
+        intersection = torch.sum(y_pred * y_true)
+        cardinality = torch.sum(y_pred + y_true)
+
+    union = cardinality - intersection
+    jaccard_score = (intersection + smooth) / (union + smooth + eps)
+    return jaccard_score
 
 
-def soft_dice_score(pred: torch.Tensor,
-                    target: torch.Tensor,
-                    smooth=1e-3,
-                    from_logits=False) -> torch.Tensor:
-    if from_logits:
-        pred = pred.sigmoid()
+def soft_dice_score(
+    y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0, eps=1e-7, dims=None
+) -> torch.Tensor:
+    """
 
-    intersection = torch.sum(pred * target)
-    union = torch.sum(pred) + torch.sum(target) + smooth
-    return 2 * intersection / union
+    :param y_pred:
+    :param y_true:
+    :param smooth:
+    :param eps:
+    :return:
+
+    Shape:
+        - Input: :math:`(N, NC, *)` where :math:`*` means any number
+            of additional dimensions
+        - Target: :math:`(N, NC, *)`, same shape as the input
+        - Output: scalar.
+
+    """
+    assert y_pred.size() == y_true.size()
+    if dims is not None:
+        intersection = torch.sum(y_pred * y_true, dim=dims)
+        cardinality = torch.sum(y_pred + y_true, dim=dims)
+    else:
+        intersection = torch.sum(y_pred * y_true)
+        cardinality = torch.sum(y_pred + y_true)
+    dice_score = (2.0 * intersection + smooth) / (cardinality + smooth + eps)
+    return dice_score
 
 
-def wing_loss(prediction: torch.Tensor, target: torch.Tensor, width=5, curvature=0.5, reduction='mean'):
+def wing_loss(
+    prediction: torch.Tensor,
+    target: torch.Tensor,
+    width=5,
+    curvature=0.5,
+    reduction="mean",
+):
     """
     https://arxiv.org/pdf/1711.06753.pdf
     :param prediction:
@@ -142,10 +184,10 @@ def wing_loss(prediction: torch.Tensor, target: torch.Tensor, width=5, curvature
     C = width - width * math.log(1 + width / curvature)
     loss[idx_bigger] = loss[idx_bigger] - C
 
-    if reduction == 'sum':
+    if reduction == "sum":
         loss = loss.sum()
 
-    if reduction == 'mean':
+    if reduction == "mean":
         loss = loss.mean()
 
     return loss
